@@ -23,16 +23,7 @@ from screenManager import screenManager, windowManager
 from trosnoth.src.gui.sound.musicManager import MusicManager
 from trosnoth.src.gui.sound.soundPlayer import SoundPlayer
 from trosnoth.src.utils import logging
-
-# timeNow is used to update things based on how much time has passed.
-# Note: on Windows, time.clock() is more precise than time.time()
-import platform
-if platform.system() == 'Windows':
-    from time import clock as timeNow
-else:
-    from time import time as timeNow    
-
-
+from trosnoth.src.utils.utils import timeNow
 
 class ApplicationExit(Exception):
     pass
@@ -43,18 +34,19 @@ class Application(object):
 
     def __init__(self, size, graphicsOptions, caption, element):
         '''Initialise the application.'''
+        self._options = (element, size, graphicsOptions, caption)
         self._initSound()
         self._makeScreenManager(element, size, graphicsOptions, caption)
-        self._initFonts()
+        self.fonts = self.screenManager.fonts
+        self.initialise()
+        self.screenManager.createInterface(element)
         self._running = False
         self.lastTime = None
 
     def __getattr__(self, attr):
         if attr == 'interface':
             return self.screenManager.interface
-
-    def _initFonts(self):
-        self.fonts = self.screenManager.fonts
+        raise AttributeError
 
     def _initSound(self):
         self.musicManager = MusicManager()
@@ -63,8 +55,26 @@ class Application(object):
     def _makeScreenManager(self, element, size, graphicsOptions, caption):
         self.screenManager = screenManager.ScreenManager(self, size,
                                                          graphicsOptions, caption)
-        self._initFonts()
-        self.screenManager.createInterface(element)
+
+    def restart(self):
+        self._initSound()
+        self._makeScreenManager(*self._options)
+        self.fonts = self.screenManager.fonts
+        self.initialise()
+        self.screenManager.createInterface(self._options[0])
+
+    def initialise(self):
+        '''
+        Provides the opportunity for initialisation by subclasses before the
+        interface element is created.
+        '''
+
+    def getFontFilename(self, fontName):
+        '''
+        May be overridden by an application in order to provide a custom font
+        location, or some other mapping from font name to filename.
+        '''
+        return fontName
 
     def run(self):
         '''Runs the application.'''
@@ -188,5 +198,3 @@ class MultiWindowApplication(Application):
     def _makeScreenManager(self, element, size, graphicsOptions, caption):
         self.screenManager = windowManager.WindowManager(self, element, size,
                                                          graphicsOptions, caption)
-        self._initFonts()
-        self.screenManager.createInterface(element)
