@@ -177,14 +177,15 @@ class SizedImage(NewImage):
 class TextImage(NewImage):
     '''Shows text at a specified screen location.'''
     
-    def __init__(self, text, font, colour=(0,128,0), bgColour=None):
+    def __init__(self, text, font, colour=(0,128,0), bgColour=None, antialias=True):
         self.colour = colour
         self.bgColour = bgColour
         self.text = text
         self.font = font
+        self.antialias = antialias
 
     def getImage(self, app):
-        return self.font.render(app, self.text, False, self.colour, self.bgColour)
+        return self.font.render(app, self.text, self.antialias, self.colour, self.bgColour)
  
 
 class Location(object):
