@@ -26,141 +26,19 @@ from twisted.internet import reactor, task
 from twisted.internet.error import CannotListenError
 from twisted.python import log
 
+from trosnoth.src.settings import DisplaySettings, SoundSettings
 from trosnoth.src.trosnothgui import interface
+from trosnoth.src.themes import Theme
 from trosnoth.src.model import universe
 from trosnoth.src.trosnothgui import defines
-from trosnoth.src.utils import logging, utils, unrepr
+from trosnoth.src.utils import logging
 from trosnoth.src.network.networkDefines import serverVersion
-from trosnoth.data import getPath, user
 
 from trosnoth.src.gui import app
-import trosnoth.src.gui.sound.musicManager as musicManager
-from trosnoth.src.gui.screenManager import screenManager
 
 from trosnoth.src.network.netman import NetworkManager
 from trosnoth.src.network.trosnothdiscoverer import TrosnothDiscoverer
-from trosnoth.src.init import initFonts, initMusic, initSound
 
-def loadSettingsFile(filename):
-    filename = getPath(user, filename)
-    try:
-        fileHandle = open(filename, 'r')
-        s = fileHandle.read()
-        d = unrepr.unrepr(s)
-        if not isinstance(d, dict):
-            d = {}
-    except IOError:
-        d = {}
-    return d
-
-class DisplaySettings(object):
-    '''
-    Stores the Trosnoth display settings.
-    '''
-
-    def __init__(self, app):
-        self.app = app
-
-        self.reset()
-
-    def reset(self):
-        # Attempt to load the settings from file.
-        data = loadSettingsFile('display')
-
-        self.size = data.get('size', defines.screenSize)
-        self.fullScreen = data.get('fullscreen', defines.fullScreen)
-        self.useAlpha = data.get('usealpha', defines.useAlpha)
-        self.fsSize = data.get('fsSize', self.size)
-
-        defines.useAlpha = self.useAlpha
-
-    def getSize(self):
-        if self.fullScreen:
-            return self.fsSize
-        else:
-            return self.size
-
-    def apply(self):
-        '''
-        Apply the current settings.
-        '''
-        defines.useAlpha = self.useAlpha
-
-        size = self.getSize()
-
-        # Don't bother changing the screen if the settings that matter haven't changed
-        if (size != self.app.screenManager.size) or (self.fullScreen !=
-                self.app.screenManager.isFullScreen()):
-            # Tell the main program to change its screen size.
-            self.app.changeScreenSize(size, self.fullScreen)
-
-    def save(self):
-        '''
-        Save the values to file.
-        '''
-        # Write to file
-        displayFilename = getPath(user, 'display')
-        displayFile = open(displayFilename, 'w')
-        displayFile.write(repr({
-            'size': self.size,
-            'fullscreen': self.fullScreen,
-            'usealpha': self.useAlpha,
-            'fsSize': self.fsSize,
-        }))
-        displayFile.close()
-        
-class SoundSettings(object):
-    '''
-    Stores the Trosnoth display settings.
-    '''
-
-    def __init__(self, app):
-        self.app = app
-
-        self.reset()
-
-    def reset(self):
-        # Attempt to load the settings from file.
-        data = loadSettingsFile('sound')
-
-        self.soundEnabled = data.get('playSound', True)
-        self.musicEnabled = data.get('playMusic', True)
-        self.musicVolume = data.get('musicVolume', 100)
-        self.soundVolume = data.get('soundVolume', 100)
-
-    def apply(self):
-        '''
-        Apply the current settings.
-        '''
-
-        if self.musicEnabled != self.app.musicManager.isMusicPlaying():
-            if self.musicEnabled:
-                self.app.musicManager.playMusic()
-            else:
-                self.app.musicManager.stopMusic()
-
-        self.app.musicManager.setVolume(self.musicVolume)
-
-        if self.soundEnabled:
-            self.app.soundPlayer.setMasterVolume(self.soundVolume / 100.)
-        else:
-            self.app.soundPlayer.setMasterVolume(0)
-
-    def save(self):
-        '''
-        Save the values to file.
-        '''
-        # Write to file
-        filename = getPath(user, 'sound')
-        dataFile = open(filename, 'w')
-        dataFile.write(repr({
-            'playSound': self.soundEnabled,
-            'playMusic': self.musicEnabled,
-            'musicVolume': self.musicVolume,
-            'soundVolume': self.soundVolume
-        }))
-        dataFile.close()
-        
 
 class Main(app.MultiWindowApplication):
     '''Instantiating the Main class will set up the game. Calling the run()
@@ -168,7 +46,7 @@ class Main(app.MultiWindowApplication):
     a game: (1) get list of clients; (2) connect to a server; (3) join the
     game.'''
 
-    def __init__(self, **kwargs):
+    def __init__(self):
         '''Initialise the game.'''
 
         self.initNetwork()
@@ -193,7 +71,7 @@ class Main(app.MultiWindowApplication):
 
         # Since we haven't connected to the server, there is no world.
         self.player = None
-           
+
     def __str__(self):
         return 'Trosnoth Main Object'
 
@@ -225,14 +103,17 @@ class Main(app.MultiWindowApplication):
 
         super(Main, self).stopping()
 
-    def _initFonts(self):
-        super(Main, self)._initFonts()
-        initFonts(self.fonts)
+    def initialise(self):
+        super(Main, self).initialise()
 
-    def _initSound(self):
-        super(Main, self)._initSound()
-        initMusic(self.musicManager)
-        initSound(self.soundPlayer)
+        # Loading the theme loads the fonts.
+        self.theme = Theme(self)
+
+    def getFontFilename(self, fontName):
+        '''
+        Tells the UI framework where to find the given font.
+        '''
+        return self.theme.getPath('fonts', fontName)
 
     def changeScreenSize(self, size, fullScreen):
         if fullScreen:
@@ -288,11 +169,7 @@ def runGame():
     if logMe:
         logging.startLogging()
     
-    # Check for command-line parameters.
-    params = {}
-    exec('params.update(%s)' % ' '.join(sys.argv[1:]))
-        
-    mainObject = Main(**params)
+    mainObject = Main()
     
     try:
         mainObject.run_twisted()
@@ -300,6 +177,10 @@ def runGame():
         if logMe:
             logging.endLogging()
 
+        if 'checkpoint' in sys.argv:
+            from trosnoth.src.utils.checkpoint import saveCheckpoints
+            saveCheckpoints()
+
 if __name__ == '__main__':
     if profileMe:
         import cProfile as profile
