@@ -17,18 +17,27 @@
 
 from math import sin, cos
 import struct
-import random
 
 import trosnoth
 
 # TODO: Get GUI code out of model
 from trosnoth.src.trosnothgui.ingame.nametag import NameTag, StarTally
+import trosnoth.src.utils.logging as logging
+from trosnoth.src.utils.checkpoint import checkpoint
+
 #####
 
 from trosnoth.src.model.unit import Unit
 from trosnoth.src.model.obstacles import JumpableObstacle, VerticalWall, GroundObstacle
 from trosnoth.src.model.map import MapLayout
+from trosnoth.src.model.universe_base import GameState
 
+from trosnoth.src.messages.shot import FireShot
+from trosnoth.src.messages.gameplay import PlayerRespawn
+from trosnoth.src.messages.transactions import DeleteUpgrade, AbandonTransaction
+# TODO: Perhaps get network code out of model
+from trosnoth.src.network.utils import compress_boolean, expand_boolean
+from trosnoth.src.network.gameplay import PlayerUpdateMsg
 
 class Player(Unit):
     '''Maintains the state of a player. This could be the user's player, a
@@ -38,11 +47,12 @@ class Player(Unit):
     is True. If the player is controlled by the network, player.local is False.
     '''
 
-    def __init__(self, universe, nick, team, id, zone, dead = False):
+    def __init__(self, app, universe, nick, team, id, zone, dead = False):
         '''Note that player instances are not local when they are created.
         The value of local is set only when the server gives us the player
         by calling self.makeLocal().'''
         Unit.__init__(self)
+        self.app = app
         
         self.universe = universe
         self.nick = nick
@@ -77,24 +87,23 @@ class Player(Unit):
         self.upgradeGauge = 0.0
         self.upgradeTotal = 0.0
         self.ghost = dead
+        # Upgrade-related
         self.upgrade = None
         self.shielded = False
         self.hasRicochet = False
         self.phaseshift = False
         self.turret = False
         self.turretOverHeated = False
+        # Shots this player has fired
         self.shots = {}
 
         # Create a nametag.
-        self.nametag = NameTag(self.nick)
-        self.starTally = StarTally(0)
+        self.nametag = NameTag(app, self.nick)
+        self.starTally = StarTally(app, 0)
         
         self.currentZone = zone
         zone.addPlayer(self)
 
-    def setAngleFacing(self, value):
-        self.angleFacing = value
-
     def setPos(self, pos):
         self.pos = pos
         # Ensure we're in the right mapBlock
@@ -115,25 +124,29 @@ class Player(Unit):
         stars = self.stars
         if self.upgrade:
             upgrade = repr(self.upgrade)
+            upgradeInUse = self.upgrade.inUse
         else:
             upgrade = None
+            upgradeInUse = False
         
         return {'pID': pID,
                 'nick': nick,
                 'team': team,
                 'dead': dead,
                 'stars': stars,
-                'upgrade': upgrade}
+                'upgrade': upgrade,
+                'upgradeInUse': upgradeInUse}
+
+    def setAngleFacing(self, value):
+        self.angleFacing = value
 
     def removeFromGame(self):
         '''Called by network client when server says this player has left the
         game.'''
         # Remove myself from the register of players.
-        if self.team.currentTransaction and self.team.currentTransaction.getNumStars(self) > 0:
-            self.team.currentTransaction.removeStars(self)
+        self.team.playerHasLeft(self)
         if self.upgrade:
             self.upgrade.clientDelete()
-        self.universe.delPlayer(self)
         
         # Remove myself from all groups I'm in.
         self.currentMapBlock.removePlayer(self)
@@ -192,7 +205,7 @@ class Player(Unit):
         
         # If the state changes and the player is not local, tell network.
         if self.local:
-            self.universe.onPlayerUpdate.execute(self)
+            self.sendPlayerUpdate()
 
     def lookAt(self, angle, thrust=None):
         '''Changes the direction that the player is looking.  angle is in
@@ -207,9 +220,9 @@ class Player(Unit):
         tempBool = self._faceRight
         self._faceRight = angle > 0
         if tempBool != self._faceRight and self.local:
-            self.universe.onPlayerUpdate.execute(self)
+            self.sendPlayerUpdate()
             
-
+    # TODO: should this only be in the interface?
     def fireShot(self):
         '''Fires a shot in the direction the player's currently looking.'''
         if not self.ghost and not self.phaseshift:
@@ -233,7 +246,9 @@ class Player(Unit):
                     
                 else:
                     self.reloadTime = self.enemyReloadTime
-                self.universe.onShotFired.execute(self, self.__getShotType())
+                self.universe.requestPlug.send(FireShot(self, self.angleFacing, self.pos[0],
+                        self.pos[1], self.__getShotType()))
+                checkpoint('Player: fire shot')
 
     def __getShotType(self):
         if self.turret:
@@ -258,7 +273,9 @@ class Player(Unit):
             deltaX = self.maxGhostVel * deltaT * sin(self.angleFacing) * self.ghostThrust
             deltaY = -self.maxGhostVel * deltaT * cos(self.angleFacing) * self.ghostThrust
 
-            self._move(deltaX, deltaY)
+            # TODO: Check for smaller resulting deltaX and deltaY due
+            #  to hitting the edge of the map.
+            self.currentMapBlock.moveUnit(self, deltaX, deltaY)
 
             # Update respawn gauge based on Time:
             if self.respawnGauge >= 0:
@@ -301,7 +318,10 @@ class Player(Unit):
                 
                 # Put the player through the floor:
                 if not isinstance(self._onGround, VerticalWall):
+                    checkpoint('Player: drop through ledge')
                     self._ignore = self._onGround
+                else:
+                    checkpoint('Player: drop off wall downwards')
                 self._onGround = None
                 
             # Now consider vertical movement.
@@ -317,6 +337,7 @@ class Player(Unit):
                    or (self._state['left'] and self._onGround.deltaPt[1] < 0):
                     self._onGround = None
                     self.fallTime = deltaT
+                    checkpoint('Player: drop off wall sideways')
             else:
                 deltaY = 0
                 deltaX = absVel * deltaT
@@ -372,60 +393,63 @@ class Player(Unit):
         distance. This method should check for collisions with bullets and the
         ground.'''
 
-        # For a ghost:
-        if self.ghost:
-                # TODO: Check for smaller resulting deltaX and deltaY due
-                #  to hitting the edge of the map.
-                self.currentMapBlock.moveUnit(self, deltaX, deltaY)
+        lastObstacle = None
+        for i in xrange(100):
+            lastPosition = self.pos
+            # Check for collisions with solid objects, and change of zone.
+            obstacle = self.currentMapBlock.moveUnit(self, deltaX, deltaY)
+            if obstacle is None:
+                break
+            if self.pos == lastPosition and obstacle == lastObstacle:
+                print 'Apparently infinite motion loop.'
+                break
+            lastObstacle = obstacle
 
-        # For a player:
+            # Ask obstacle where the player should end up.
+            targetPt = obstacle.finalPosition(self.pos, deltaX, deltaY)
+            
+            if targetPt == None:
+                break
+
+            deltaX = targetPt[0] - self.pos[0]
+            deltaY = targetPt[1] - self.pos[1]
         else:
-            while True:
-                # Check for collisions with solid objects, and change of zone.
-                obstacle = self.currentMapBlock.moveUnit(self, deltaX, deltaY)
-                if obstacle is not None:
-                    # Ask obstacle where the player should end up.
-                    targetPt = obstacle.finalPosition(self.pos, deltaX, deltaY)
-                    
-                    if targetPt == None:
-                        break
-
-                    deltaX = targetPt[0] - self.pos[0]
-                    deltaY = targetPt[1] - self.pos[1]
-                    
-                else:
-                    break
-                
-            # The following if statement will perform calculations for setting
-            #  the current obstacle if any of the following conditions are met:
-            # (1) Player is not currently on the ground.
-            # (2) Player is currently on the ground and has hit another obstacle.
-            # (3) Player was on the ground but has walked off the end.
-            if not isinstance(self._onGround, GroundObstacle) or obstacle or \
-                               not self._onGround.inBounds(self.pos):
-                if isinstance(obstacle, JumpableObstacle):
-                    if isinstance(self._onGround, GroundObstacle) and \
+            print 'Very very bad thing: motion loop did not terminate after 100 iterations'
+            
+        # The following if statement will perform calculations for setting
+        #  the current obstacle if any of the following conditions are met:
+        # (1) Player is not currently on the ground.
+        # (2) Player is currently on the ground and has hit another obstacle.
+        # (3) Player was on the ground but has walked off the end.
+        if not isinstance(self._onGround, GroundObstacle) or obstacle:
+            if isinstance(obstacle, JumpableObstacle):
+                if isinstance(self._onGround, GroundObstacle) and \
                        isinstance(obstacle, VerticalWall):
-                        # Running into a vertical wall while on the ground.
-                        # Don't attach self to it.
-                        pass
-                    else:
-                        self._onGround = obstacle
+                    # Running into a vertical wall while on the ground.
+                    # Don't attach self to it.
+                    pass
                 else:
-                    self._onGround = None
+                    self._onGround = obstacle
+            else:
+                self._onGround = None
+            if obstacle:
+                # Hit an obstacle. Process change to velocity etc.
                 if obstacle:
-                    # Hit an obstacle. Process change to velocity etc.
-                    if obstacle:
-                        obstacle.hitByPlayer(self)
+                    obstacle.hitByPlayer(self)
+        elif isinstance(self._onGround, GroundObstacle):
+            self._onGround, self.pos = self._onGround.checkBounds(self.pos)
 
-            # Check for zone tag.
-            if self.currentZone.orbOwner != self.team and self.local:
-                self.universe.checkTag(self)
+        # Check for zone tag.
+        if self.currentZone.orbOwner != self.team and self.local:
+            self.universe.checkTag(self)
 
         self._ignore = None
 
     def changeZone(self, newZone):
-        self.currentZone.removePlayer(self)
+        try:
+            self.currentZone.removePlayer(self)
+        except:
+            logging.writeException()
         newZone.addPlayer(self)
         self.currentZone = newZone
 
@@ -436,9 +460,12 @@ class Player(Unit):
             if self.team.currentTransaction and \
                self.team.currentTransaction.contributions.has_key(self):
                 self.team.currentTransaction.removeStars(self)
-        self.universe.updateStars(self, 0)
+        self.team.starCountDecreased()
+        self.stars = 0
+        self.updateNametag()
         if self.upgrade:
             self.upgrade.clientDelete()
+        checkpoint('Player: died')
 
     def respawn(self):
         self._onGround = None
@@ -450,6 +477,7 @@ class Player(Unit):
             self.currentMapBlock = self.universe.zoneBlocks[i][j]
         except IndexError:
             raise IndexError, "You're off the map!"
+        checkpoint('Player: respawned')
         
     def updateNametag(self):
         self.starTally.setStars(self.stars)
@@ -459,13 +487,14 @@ class Player(Unit):
         if self.upgrade:
             print "%s's Upgrade Gone" % self.nick
             self.upgrade.clientDelete()
+        else:
+            print 'Attempt to delete an upgrade which does not exist'
 
     def destroyShot(self, sId):
         try:
             del self.shots[sId]
         except KeyError:
-#            logging.writeException()
-            pass
+            logging.writeException()
 
     def addShot(self, shot):
         self.shots[shot.id] = shot
@@ -473,15 +502,12 @@ class Player(Unit):
     def canRequestUpgrade(self, upgradeClass):
         if self.team.currentTransaction != None:
             return False, 'Transaction already in play'
-        
-        elif self.upgrade != None:
+        if self.upgrade != None:
             return False, 'You already have an upgrade'
-        
-        elif self.team.teamStars < upgradeClass.requiredStars:
-            return False, 'Your team has insufficient stars'
-        
-        elif self.ghost:
+        if self.ghost:
             return False, 'You are dead! Can\'t purchase an upgrade'
+        if self.universe.getTeamStars(self.team) < upgradeClass.requiredStars:
+            return False, 'Your team has insufficient stars'
         return True, ''
 
     def hasThisManyStars(self, numStars):
@@ -519,9 +545,10 @@ class Player(Unit):
         assert self.canUseUpgrade()[0]
         self.upgrade.requestUse()
         self.upgradeGauge = self.upgradeTotal = self.upgrade.timeRemaining
-        self.universe.onUpgradeUsed.execute(self)
 
     def canRespawn(self):
+        if self.universe.gameState == GameState.PreGame:
+            return False, 'Game has not started'
         if not self.ghost:
             return False, 'Already Alive'
         elif self.respawnGauge > 0:
@@ -533,13 +560,81 @@ class Player(Unit):
 
     def requestRespawn(self):
         '''Called by the menu when a player wishes to respawn'''
-        assert self.canRespawn()
-        self.universe.onPlayerRespawn.execute(self)
+        # TODO: consider removing this assertion
+        #       if we move to the three-plug universe approach
+        assert self.canRespawn()[0]
+        self.universe.requestPlug.send(PlayerRespawn(self, self.currentZone))
 
+    # TODO: this is a bit out of place - you don't abandon a player
     def abandon(self):
-
-        if self.upgrade:
-            self.universe.onUpgradeReqDel.execute(self)
-        elif self.team.currentTransaction and \
+        if self.upgrade is not None:
+            self.universe.requestPlug.send(DeleteUpgrade(self, self.upgrade))
+        elif self.team.currentTransaction is not None and \
              self.team.currentTransaction.purchasingPlayer == self:
-            self.universe.onAbandonTransaction.execute(self.team)
+            self.universe.requestPlug.send(AbandonTransaction(self.team.currentTransaction, "request"))
+
+
+    def sendPlayerUpdate(self):
+        '''Sends the server an update of the player's current state. Player
+        should be a locally-controlled player - that is, a player created using
+        the join() method. An update only needs to be sent when the player
+        changes state (which keys are being pressed), because actual position
+        of the player is predicted by each client.'''
+        values = compress_boolean( (self._state['left'], self._state['right'],
+                self._state['jump'], self._state['down'],
+                self.upgrade is not None, self.ghost ) )
+        
+        msg = PlayerUpdateMsg(self.id, self.pos[0], self.pos[1],
+                self.yVel, self.angleFacing, self.ghostThrust,
+                str(values))
+        self.universe.requestPlug.send(msg)
+
+    def gotPlayerUpdate(self, msg):
+        if not self.local:
+            values = expand_boolean(getattr(msg, 'keys'))
+            opts = ['left', 'right', 'jump', 'down']
+            
+            for i in range(4):
+                self.updateState(opts[i], bool(values[i]))
+                
+            hasUpgrade = values[4]
+            ghost = values[5]
+            
+            if (hasUpgrade and not self.upgrade):
+                print 'Server tells me I have an upgrade. I don\'t think I do'
+            elif (not hasUpgrade and self.upgrade):
+                print 'Server tells me I do not have an upgrade. I think I do'
+                #self.requestUpgrades()
+                
+            if ghost != self.ghost:
+                if self.ghost:
+                    self.respawn()
+                else:
+                    self.die()
+            
+            self._positionUpdate((msg.xPos, msg.yPos),
+                    msg.yVel, angle=msg.angle, thrust=msg.ghostThrust)
+
+    ##
+    # Force-update our position
+    def _positionUpdate(self, pos, yVel=None, angle=None, thrust=None):
+        if not self.local:
+            if yVel is not None:
+                self.yVel = yVel
+
+            if angle is not None:
+                self.lookAt(angle, thrust)
+
+            self.setPos((pos[0], pos[1]))
+
+    def incrementStars(self):
+        if self.stars < max:
+            self.stars += 1
+            self.updateNametag()
+        print '%s - %d stars'
+        
+    def subtractStars(self, stars):
+        assert self.stars >= stars, "Player: Tried to subtract %d stars from %d" % (stars, self.stars)
+        self.stars -= stars
+        self.team.starCountDecreased()
+        self.updateNametag()
