@@ -18,41 +18,44 @@
 '''universe.py - defines anything that has to do with the running of the
 universe. This includes players, shots, zones, and the level itself.'''
 
-from twisted.internet import reactor
 import pygame
 
-import trosnoth
 from trosnoth.src.utils.utils import timeNow
 import trosnoth.src.utils.logging as logging
+from trosnoth.src.utils.checkpoint import checkpoint
 import trosnoth.src.model.modes as modes
 from trosnoth.src.model.universe_base import GameState
+import trosnoth.src.model.transaction as transactionModule
+from trosnoth.src.model.upgrades import upgradeOfType
 
-# TODO: Colours should not be in model
-from trosnoth.src.trosnothgui.ingame.colours import *
-from trosnoth.src.gui.framework.utils import makeImage
-from trosnoth.src.trosnothgui.ingame.nametag import NameTag
-
-from trosnoth.data import fonts, getPath
-
-# TODO; remove isinstances so we can remove some of these:
-from trosnoth.src.model.obstacles import *
-from trosnoth.src.model.mapblocks import MapBlockDef
+# TODO: remove isinstances so we can remove some of these:
 from trosnoth.src.model.map import MapLayout, MapState
+from trosnoth.src.model.mapblocks import BodyMapBlock
 from trosnoth.src.model.shot import Shot, GrenadeShot
 from trosnoth.src.model.player import Player
 from trosnoth.src.model.team import Team
 
-from trosnoth.src.utils.event import Event
 
 # Component message passing
 from trosnoth.src.utils.components import Component, handler, Plug
-from trosnoth.src.messages.chat import *
-from trosnoth.src.messages.connection import *
-from trosnoth.src.messages.game import *
-from trosnoth.src.messages.gameplay import *
-from trosnoth.src.messages.players import *
-from trosnoth.src.messages.shot import *
-from trosnoth.src.messages.transactions import *
+from trosnoth.src.messages.chat import Chat, OwnChat
+from trosnoth.src.messages.game import (SetCaptain, StartingSoon, TeamIsReady,
+        GameOver, SetGameMode, SetTeamName)
+from trosnoth.src.messages.gameplay import (TaggedZone, PlayerKilled, KillShot,
+        PlayerRespawn)
+from trosnoth.src.messages.players import AddPlayer, RemovePlayer
+from trosnoth.src.messages.shot import ShotFired, ShotGone
+from trosnoth.src.messages.grenade import GrenadeNowExists, GrenadeGone
+from trosnoth.src.messages.transactions import (TransactionChanged,
+        StartedTransaction, TransactionComplete, AbandonTransaction,
+        AddedStars, UseUpgrade, DeleteUpgrade)
+from trosnoth.src.messages.validation import NotifyUDPStatus
+from trosnoth.src.network.chat import ChatFromServerMsg
+from trosnoth.src.network.shot import ShotFiredMsg
+from trosnoth.src.network.gameplay import PlayerUpdateMsg
+from trosnoth.src.network.game import GameStartMsg
+from trosnoth.src.network.players import AddPlayerMsg
+from trosnoth.src.network.transactions import StartedTransactionMsg
 
 
 # FIXME: there's the potential for some things to require an interface
@@ -62,10 +65,7 @@ from trosnoth.src.messages.transactions import *
 def init():
     '''Initialises anything that's needed by this module. This function should
     only be called after pygame.init() and after the video mode has been set.'''
-    # Load the name font.
     pygame.font.init()
-    NameTag.nameFont = pygame.font.Font(getPath(fonts, 'FreeSans.ttf'), 20)
-
             
 class Universe(Component):
     '''Universe(halfMapWidth, mapHeight)
@@ -77,9 +77,22 @@ class Universe(Component):
     halfPlayerSize = (10, 19)
     halfShotSize = (5, 5)
 
-    actionPlug = Plug()    
+    # actionPlug is a sending plug
+    # Things are sent to it after something happens
+    # This may in turn be picked up by a network or a GUI
+    actionPlug = Plug()
 
-    def __init__(self, halfMapWidth, mapHeight):
+    # requestPlug is a sending plug
+    # Things that this universe thinks should happen
+    # should be sent to it
+    requestPlug = Plug()
+
+    # orderPlug is a receiving plug
+    # When an object wants to order the universe to do something,
+    # they should send a message to this plug
+    orderPlug = Plug()
+
+    def __init__(self, app, halfMapWidth, mapHeight):
         '''
         halfMapWidth:   is the number of columns of zones in each team's
                         territory at the start of the game. There will always
@@ -95,6 +108,8 @@ class Universe(Component):
                         mean that the column has mapHeight + 2 zones in it.'''
 
         super(Universe, self).__init__()
+        self.app = app
+
         # Initialise
         self.gameState = GameState.PreGame
         self.startTime = 0
@@ -103,79 +118,14 @@ class Universe(Component):
         self.teamWithId = {'\x00' : None}
 
         # Create Teams:
-        player1img = makeImage('Blue.png')
-        teamAShot = makeImage('blueShot.bmp')
-        player1img.set_colorkey((0,0,0))
-        team1orb = makeImage('blueOrb.png')
-        player2img = makeImage('Red.png')
-        teamBShot = makeImage('redShot.bmp')
-        player2img.set_colorkey((0,0,0))
-        team2orb = makeImage('redOrb.png')
-
-        self.neutralOrb = makeImage('greyOrb.png')
-
-        # This is an active universe.
-
-        self.teams = (Team(team1bg,
-                           team1msg,
-                           team1chat,
-                           team1Mn_zone,
-                           team1Mn_mk,
-                           team1Mn_pl,
-                           team1Mn_gh,
-                           player1img,
-                           teamAShot,
-                           team1orb,
-                           'A'),
-                      Team(team2bg,
-                           team2msg,
-                           team2chat,
-                           team2Mn_zone,
-                           team2Mn_mk,
-                           team2Mn_pl,
-                           team2Mn_gh,
-                           player2img,
-                           teamBShot,
-                           team2orb,
-                           'B'))
-##        else:
-##            # This universe will not actually be displayed (created by server)
-##            self.teams = (Team(colours.team1bg,
-##                               colours.team1msg,
-##                               None,
-##                               'A'),
-##                          Team(colours.team2bg,
-##                               colours.team2msg,
-##                               None,
-##                               'B'))
-##            
+        self.teams = (
+            Team(self, 'A'),
+            Team(self, 'B'),
+        )
         Team.setOpposition(self.teams[0], self.teams[1])
         
         for t in self.teams:
-            self.teamWithId[t.id] = t
-
-        self.onZoneTagged = Event()
-        self.onPlayerKilled = Event()
-        self.onShotDestroyed = Event()
-        self.onShotFired = Event()
-        self.onZoneChanged = Event()
-        self.onPlayerUpdate = Event()
-        self.onPlayerRespawn = Event()
-        self.onUpgradeReqUse = Event()
-        self.onUpgradeUsed = Event()
-        self.onAbandonTransaction = Event()
-        self.onUpgradeReqDel = Event()
-        self.onUpgradeDestroyed = Event()
-        self.onGrenadeExploded = Event()
-        self.onTurretBegin = Event()
-        self.onTurretEnd = Event()
-        self.onMinimapBegin = Event()
-        self.onMinimapEnd = Event()
-
-        self.onShotKill = Event()
-        self.onGrenadeKill = Event()
-        self.onGrenadeExists = Event()
-        
+            self.teamWithId[t.id] = t        
                 
         # Set up zones
         self.zoneWithDef = {}
@@ -192,7 +142,9 @@ class Universe(Component):
         self.gameOverTime = None
         self.gameModeController = modes.GameMode(Shot, Player, GrenadeShot)
 
-    def setGameMode(self, mode):
+    @handler(SetGameMode, orderPlug)
+    def setGameMode(self, msg):
+        mode = msg.gameMode
         try:
             getattr(self.gameModeController, mode)()
             print 'Client: GameMode is set to ' + mode
@@ -201,6 +153,10 @@ class Universe(Component):
         except TypeError:
             print 'No such gameMode'
 
+    @handler(SetTeamName, orderPlug)
+    def setTeamName(self, msg):
+        msg.team.teamName = msg.name
+
     def setTimeLeft(self, time):
         # TODO: could do some validation here.
         pass
@@ -213,25 +169,36 @@ class Universe(Component):
         elif self.gameState == GameState.Ended:
             return self.startTime + self.gameDuration - self.gameOverTime
 
-
-    def gameOver(self):
+    @handler(GameOver, orderPlug)
+    def gameOver(self, msg):
+        winningTeam, timeOver = msg.winningTeam, msg.timeOver
         self.gameOverTime = timeNow()
         self.gameState = GameState.Ended
+        self.actionPlug.send(msg)
 
-    def gameStart(self, time):
+    @handler(GameStartMsg, orderPlug)
+    def gameStart(self, msg):
+        time = msg.timeLimit
         if self.gameState == GameState.PreGame:
             self.gameDuration = time
             self.startTime = timeNow()
             self.gameState = GameState.InProgress
+            self.actionPlug.send(msg)
 
-    def addPlayer(self, nick, team, playerId, zone, dead = True):
+    @handler(AddPlayerMsg, orderPlug)
+    def addPlayer(self, msg):
+        team = self.teamWithId[msg.teamId]
+        zone = self.zoneWithId[msg.zoneId]
+
+        # Create the player.
+        nick = msg.nick.decode()
         
-        player = Player(self, nick, team, playerId, zone,
-                        dead = True)
+        player = Player(self.app, self, nick, team, msg.playerId, zone,
+                        msg.dead)
 
         #Add this player to this universe.
         self.players.add(player)
-        self.playerWithId[playerId] = player
+        self.playerWithId[msg.playerId] = player
 
         # Make sure player knows its zone
         i, j = MapLayout.getMapBlockIndices(*player.pos)
@@ -243,9 +210,11 @@ class Universe(Component):
         
         self.actionPlug.send(AddPlayer(player))
 
-    def delPlayer(self, player):
-        '''Removes the specified player from this universe. Should be called
-        by the network client when the server sends an appropriate message.'''
+    @handler(RemovePlayer, orderPlug)
+    def delPlayer(self, msg):
+        '''Removes the specified player from this universe.'''
+        player = msg.player
+        player.removeFromGame()
         self.players.remove(player)
         try:
             del self.playerWithId[player.id]
@@ -253,6 +222,8 @@ class Universe(Component):
             logging.writeException()
             pass
         
+        self.actionPlug.send(msg)
+        
     def tick(self, deltaT):
         '''Advances all players and shots to their new positions.'''
         # Update the player and shot positions.
@@ -261,22 +232,60 @@ class Universe(Component):
         map(update, list(self.players))
         map(update, list(self.grenades))
 
-        # Tell the network about tagged zones
         for zone in self.zonesToReset:
             zone.taggedThisTime = False
         self.zonesToReset = []
+        
+        self.checkForResult()
+        
+    def checkForResult(self):
+        if self.gameState == GameState.InProgress and self.gameDuration > 0:
+            # Check first for timeout
+            if self.getTimeLeft() <= 0:
+                if self.teams[0].numOrbsOwned > self.teams[1].numOrbsOwned:
+                    self.requestPlug.send(GameOver(self.teams[0], True))
+                    checkpoint('Universe: Out of time, blue wins')
+                elif self.teams[1].numOrbsOwned > self.teams[0].numOrbsOwned:
+                    self.requestPlug.send(GameOver(self.teams[1], True))
+                    checkpoint('Universe: Out of time, red wins')
+                else:
+                    self.requestPlug.send(GameOver(None, False))
+                    checkpoint('Universe: Out of time, game drawn')
+                return
+                    
+            # Now check for an all zones win.
+            team2Wins = self.teams[0].isLoser()
+            team1Wins = self.teams[1].isLoser()
+            if team1Wins and team2Wins:
+                # The extraordinarily unlikely situation that all
+                # zones have been neutralised in the same tick
+                self.requestPlug.send(GameOver(None, False))
+                checkpoint('Universe: Draw due to all zones neutralised')
+            elif team1Wins:
+                self.requestPlug.send(GameOver(self.teams[0], False))
+                checkpoint('Universe: Zones captured, blue wins')
+            elif team2Wins:
+                self.requestPlug.send(GameOver(self.teams[1], False))
+                checkpoint('Universe: Zones captured, red wins')
 
     def validateTurretVal(self, player, turretVal):
         return player.turret == turretVal
 
-    def shotFired(self, id, team, pos, angle, player, turret, ricochet):
-        '''This should be called by the networkClient object when the server
-        indicates that a player has fired a shot.'''
+    @handler(ShotFiredMsg, orderPlug)
+    def shotFired(self, msg):
+        '''A player has fired a shot.'''
+        player = self.playerWithId[msg.playerId]
+        team = player.team
+        pos = (msg.xpos, msg.ypos)
+        turret = msg.type in ('T', 'G')
+        ricochet = msg.type == 'R'
         #assert self.validateTurretVal(player, turret)
-        
-        self.shots.add(Shot(id, team, pos, angle, player, turret, ricochet))
+        shot = Shot(msg.shotId, team, pos, msg.angle, player, turret, ricochet)
+        self.shots.add(shot)
+        self.actionPlug.send(ShotFired(player, shot))
 
     def removeShot(self, shot):
+        self.actionPlug.send(ShotGone(shot))
         try:
             self.shots.remove(shot)
         except KeyError:
@@ -359,27 +368,36 @@ class Universe(Component):
 
             if numTaggers > numDefenders or numTaggers > 3:
                    
-                if numDefenders > 3 and \
-                   zone.checkMore(tagger.team.opposingTeam):
+                if (numDefenders > 3 and 
+                        zone.checkForOpposingTag(tagger.team)):
                     # The other team has also tagged it
                     if zone.orbOwner != None:
-                        self.onZoneTagged.execute(zone, None, None)
+                        self.requestPlug.send(TaggedZone(zone, None))
+                        checkpoint('Universe: two teams tag same zone')
                 else:
                     # This team is the only to have tagged it
-                    self.onZoneTagged.execute(zone, tagger, tagger.team)
+                    self.requestPlug.send(TaggedZone(zone, tagger))
+                    checkpoint('Universe: zone tagged')
 
                 zone.taggedThisTime = True
                 self.zonesToReset.append(zone)
+            else:
+                checkpoint('Universe: failed attempt to tag zone')
 
-    def killPlayer(self, player, killer, shot):
+    @handler(PlayerKilled, orderPlug)
+    def killPlayer(self, msg):
         '''Turns a player into a ghost at the given location, crediting the
         killer with a star'''
+        player, killer, shot = msg.target, msg.killer, msg.shot
         if shot:
             print killer.nick + ' killed ' + player.nick
         else:
             print killer.nick + ' killed turreted player ' + player.nick + \
                   ' by tagging the zone'
 
+        # Credit the killer
+        killer.incrementStars()
+
         # Remove player from its zone as a living player
         player.currentZone.removePlayer(player)
         # Turn them into a ghost
@@ -388,11 +406,19 @@ class Universe(Component):
         player.currentZone.addPlayer(player)
         if shot:
             shot.kill()
+            
+        self.actionPlug.send(msg)
 
+    @handler(TaggedZone, orderPlug)
+    def tagged(self, msg):
+        msg.zone.tag(msg.player)
+        self.actionPlug.send(msg) 
 
-    def respawn(self, ghost, zone):
+    @handler(PlayerRespawn, orderPlug)
+    def respawn(self, msg):
         '''Brings a player back into the land of the living'''
         # Remove player from its zone as a ghost
+        ghost, zone = msg.player, msg.zone
         ghost.currentZone.removePlayer(ghost)
         ghost.currentZone = zone
         ghost.respawn()
@@ -400,6 +426,24 @@ class Universe(Component):
         # Add player back to the zone as a living player
         ghost.currentZone.addPlayer(ghost)
 
+        self.actionPlug.send(PlayerRespawn(ghost, zone))        
+
+    # A hacky way of updating contributions if we lost information somewhere.
+    def updateContributions(self, transaction, contributions):
+        transaction.contributions = contributions
+        self.actionPlug.send(TransactionChanged(transaction))
+
+
+    @handler(StartedTransactionMsg, orderPlug)
+    def startTransaction(self, msg):
+        team = self.teamWithId[msg.teamId]
+        player = self.playerWithId[msg.playerId]
+        upgrade = upgradeOfType[msg.upgradeType]
+        transaction = transactionModule.Transaction(team, player,
+                                                       upgrade, msg.timeLeft)
+        self.actionPlug.send(StartedTransaction(transaction))
+        transaction.addStars(player, msg.numStars)
+
     def validateTransaction(self, team, update, numStars):
         if (not team.currentTransaction and update != 'a') or \
            (update == 's' and not team.currentTransaction.totalStars == numStars):
@@ -411,16 +455,26 @@ class Universe(Component):
             return False
         return True
 
-    
-    def addStarsToTransaction(self, team, player, stars):
+    @handler(AddedStars, orderPlug)
+    def addStarsToTransaction(self, msg):
+        team = msg.team
+        player = msg.player
+        stars = msg.numStars
         team.currentTransaction.addStars(player, stars)
+        self.actionPlug.send(msg)
 
-    def abandonTransaction(self, team):        
-        team.currentTransaction.abandon()
-
-    def completeTransaction(self, team):
-        team.currentTransaction.complete()   
+    @handler(AbandonTransaction, orderPlug)
+    def abandonTransaction(self, msg):
+        # TODO: shouldn't need this check:
+        
+        # If the transaction doesn't exist, don't try and
+        # abandon it (it was likely already abandoned)
+        if msg.transaction is None:
+            return
+        msg.transaction.abandon(msg.reason)
+        self.actionPlug.send(msg)
 
+    
     def mapBlockWidth(self, blockClass):
         '''Returns the width in pixels of a map block of the given class.'''
         if issubclass(blockClass, BodyMapBlock):
@@ -428,26 +482,90 @@ class Universe(Component):
         else:
             return MapLayout.zoneInterfaceWidth
 
-
-    def completeTransaction(self, team):
+    @handler(TransactionComplete, orderPlug)
+    def completeTransaction(self, msg):
         '''Completes the team's transaction, allocating the player who purchased
         it the relevant upgrade'''
-        if team.currentTransaction == None:
-            return
-        team.currentTransaction.complete()
-        team.currentTransaction = None
+        if msg.team.currentTransaction == None:
+            transaction = transactionModule.Transaction(msg.team, msg.player,
+                                                       msg.upgradeType, 30)
+            msg.team.currentTransaction = transaction
+            transaction.addStars(msg.player, msg.upgradeType.requiredStars)
+            msg.transaction = transaction
+        
+        msg.team.currentTransaction.complete()
+        self.actionPlug.send(msg)
         print 'Transaction Completed'
 
-    def updateStars(self, player, stars):
-        oldNum = player.stars
-        diff = stars - oldNum
-        player.team.updateTeamStars(diff)
-        player.stars = stars
-        player.updateNametag()
-
     def addGrenade(self, grenade):
         self.grenades.add(grenade)
+        self.actionPlug.send(GrenadeNowExists(grenade))
 
     def removeGrenade(self, grenade):
         self.grenades.remove(grenade)
+        self.actionPlug.send(GrenadeGone(grenade))
 
+    @handler(SetCaptain, orderPlug)
+    def setCaptain(self, msg):
+        captain = msg.captain
+        captain.team.captain = captain
+        self.actionPlug.send(msg)
+
+    @handler(TeamIsReady, orderPlug)
+    def teamReady(self, msg):
+        player = msg.player
+        player.team.ready = True
+        self.actionPlug.send(TeamIsReady(player))
+
+    @handler(KillShot, orderPlug)
+    def shotDestroyed(self, msg):
+        if msg.shot is not None:
+            msg.shot.kill()
+
+    @handler(UseUpgrade, orderPlug)
+    def useUpgrade(self, msg):
+        msg.player.upgrade.clientUse()
+        self.actionPlug.send(msg)
+
+    @handler(DeleteUpgrade, orderPlug)
+    def destroyUpgrade(self, msg):
+        msg.player.deleteUpgrade()
+        self.actionPlug.send(msg)
+
+    @handler(PlayerUpdateMsg, orderPlug)
+    def gotPlayerUpdate(self, msg):
+        # Receive a player update
+        try:
+            player = self.playerWithId[msg.playerId]
+        except:
+            # Mustn't have that info yet
+            return
+        player.gotPlayerUpdate(msg)
+
+    # Passing some chat messages on to the UI
+    @handler(OwnChat, orderPlug)
+    def gotOwnChat(self, msg):
+        self.actionPlug.send(msg)
+
+    @handler(Chat, orderPlug)
+    def gotChat(self, msg):
+        self.actionPlug.send(msg)
+
+    @handler(NotifyUDPStatus, orderPlug)
+    def gotUDPNotification(self, msg):
+        self.actionPlug.send(msg)
+
+    @handler(ChatFromServerMsg, orderPlug)
+    def gotServerChat(self, msg):
+        self.actionPlug.send(msg)
+
+    @handler(StartingSoon, orderPlug)
+    def gotStartingSoon(self, msg):
+        self.actionPlug.send(msg)
+        
+    def getTeamStars(self, team):
+        total = 0
+        for p in self.players:
+            if p.team == team:
+                total += p.stars
+        return total
