@@ -15,17 +15,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-import twisted.internet
 from twisted.internet import defer, reactor
-from twisted.internet.protocol import ClientFactory, DatagramProtocol
-from twisted.protocols.basic import Int32StringReceiver
-from random import choice, random
-from math import pi
+from twisted.python.failure import Failure
 import os
-import re
 
 import struct
-import marshal
 
 from trosnoth.src.utils.unrepr import unrepr
 from trosnoth.src.utils.event import Event
@@ -45,17 +39,13 @@ from trosnoth.src.network.validation import *
 from trosnoth.src.network.gameplay import *
 from trosnoth.src.network.map import *
 from trosnoth.src.network.transactions import *
-from trosnoth.src.network.utils import *
 
 from trosnoth.src.utils import logging
-from trosnoth.src.model.upgrades import Turret, upgradeOfType, upgradeNames
+from trosnoth.src.model.upgrades import Turret, upgradeOfType
 from trosnoth.src.model.universe_base import GameState
-import trosnoth.src.model.transaction as transactionModule
 
 from trosnoth.src.utils.utils import timeNow
 
-from trosnoth.src.trosnothgui.ingame import colours
-
 from trosnoth.data import getPath, user
 
 # Component message passing
@@ -66,9 +56,12 @@ from trosnoth.src.messages.game import *
 from trosnoth.src.messages.gameplay import *
 from trosnoth.src.messages.players import *
 from trosnoth.src.messages.shot import *
+from trosnoth.src.messages.grenade import *
 from trosnoth.src.messages.transactions import *
 from trosnoth.src.messages.validation import NotifyUDPStatus
 
+from trosnoth.src.network.networkMessageTranslator import MessageTranslator_N2U
+
 from trosnoth.src.model.player import Player
 
 # TODO: Define some exception classes and use them instead of Exception
@@ -306,9 +299,16 @@ class ClientNetHandler(Component):
 class Client(Component):
 
     gamePlug = Plug()
-    viewPlug = Plug()    
+
+    # Be ordered to do something    
+    orderPlug = Plug()
+
+    # Tell the universe to do something
+    worldPlug = Plug()
+    tempWorldPlug = Plug() # For translator
 
     def __init__(self, app):
+        self.__createMessagePassOn()
         Component.__init__(self)
         self.app = app
 
@@ -316,11 +316,32 @@ class Client(Component):
 
         self.onClosed = Event()     # reason
     
-        self.layoutDatabase = mapLayout.LayoutDatabase()
+        self.layoutDatabase = mapLayout.LayoutDatabase(app)
 
         self.cancelingPlayers = []
         self.waitingPlayers = {}
 
+    def __createMessagePassOn(self):
+        messagesToPassOn = (TaggedZoneMsg, KilledMsg, RespawnMsg, NotifyUDPStatus, AddPlayerMsg, GameStartMsg,
+                            RemovePlayerMsg, ShotFiredMsg, TeamIsReadyMsg, SetCaptainMsg, KillShotMsg, PlayerUpdateMsg,
+                            StartedTransactionMsg, DeleteUpgradeMsg, AddedStarsMsg, AbandonTransactionMsg,
+                            UseUpgradeMsg, TransactionCompleteMsg, ChatMsg, ChatFromServerMsg, StartingSoonMsg,
+                            GameOverMsg, SetGameModeMsg, SetTeamNameMsg)
+        
+        for messageType in messagesToPassOn:
+            string = 'handle_%s' % (messageType.__name__,)
+            @handler(messageType, self.gamePlug)
+            def fn(self, msg):
+                self.tempWorldPlug.send(msg)
+            setattr(self, string, fn)
+
+    # TODO: get rid of this
+    def positionUpdate(self, pId, pos, yVel=None,
+            angle=None, thrust=None):
+        player = self.world.playerWithId[pId]
+        player._positionUpdate(pos, yVel, angle, thrust)                
+
+
     def join(self, nick, team):
         '''Creates a player for you, and joins the game with the specified nick
         and on the specified team. Returns a deferred, whose callback is
@@ -329,7 +350,10 @@ class Client(Component):
         time it's called. This is so that bots can be supported.'''
         result = defer.Deferred()
 
-        self.gamePlug.send(JoinRequestMsg(team.id, nick.encode()))
+        if team is None:
+            self.gamePlug.send(JoinRequestMsg("x", 1, nick.encode()))
+        else:
+            self.gamePlug.send(JoinRequestMsg(team.id, 0, nick.encode()))
 
         # TODO: Maybe check if we've already got a waiting request for this
         # team and nick.
@@ -357,151 +381,110 @@ class Client(Component):
         self._leave(player.id)
 
         # TODO: Remove all player's stars from any current transaction
-    
-    def sendPlayerUpdate(self, player):
-        '''Sends the server an update of the player's current state. Player
-        should be a locally-controlled player - that is, a player created using
-        the join() method. An update only needs to be sent when the player
-        changes state (which keys are being pressed), because actual position
-        of the player is predicted by each client.'''
-        i, j = player.currentMapBlock.defn.indices
-        
-        values = compress_boolean( (player._state['left'], player._state['right'],
-                player._state['jump'], player._state['down'],
-                player.upgrade is not None, player.ghost ) )
-        
-        msg = PlayerUpdateMsg(player.id, player.pos[0], player.pos[1],
-                player.yVel, player.angleFacing, player.ghostThrust,
-                str(values))
 
+    @handler(PlayerUpdateMsg, orderPlug)    
+    def sendPlayerUpdate(self, msg):
         self.gamePlug.send(msg)
-        return defer.succeed(None)
 
-    def positionUpdate(self, pId, pos, yVel=None,
-            angle=None, thrust=None):
-        player = self.world.playerWithId[pId]
-        if not player.local:
-            if yVel is not None:
-                player.yVel = yVel
+    # The next few handlers are purely to suppress harmless warnings.
+    # With the unification of server and client universes, there will
+    # for the time being, be some messages which the client universe
+    # generates that the network client should just ignore.
+    # Ultimately, the client universe will probably not have anything
+    # plugged into that plug, so warnings won't be generated.
+    #
+    # Or so I think...
+    @handler(GameOver, orderPlug)    
+    def sendGameOver(self, msg):
+        pass
 
-            if angle is not None:
-                player.lookAt(angle, thrust)
 
-            player.setPos((pos[0], pos[1]))
-    
-    def sendShotFired(self, player, type):
+    @handler(FireShot, orderPlug)
+    def sendShotFired(self, msg):
         '''Sends the server a message indicating that the player wants to
         fire a shot, and giving the player's current location and viewing
         angle. Client does not assume shot has been fired until it receives
         a response from the server.'''
-        self.gamePlug.send(FireShotMsg(player.id, player.angleFacing, player.pos[0],
-                player.pos[1], type))
-        
-        return defer.succeed(None)
-    
-    def sendGrenadeExploded(self, grenade):
-        self.gamePlug.send(GrenadeExplodedMsg(grenade.player.id, grenade.gr.pos[0], grenade.gr.pos[1]))
-    
-    def receiveShotFired(self, pos, angle, pId, sId, turret, ricochet):
-        '''Tells the universe object that a shot has been fired.'''
-        player = self.world.playerWithId[pId]
-        team = player.team
-        if self.world is not None:
-##            if not self.world.validateTurretVal(player, turret):
-##                self.requestUpgrades()
-##                return
-            self.world.shotFired(sId, team, pos, angle, player, turret, ricochet)
-            # A bit messy
-            self.viewPlug.send(ShotFired(player, self.world.shotWithId(pId, sId)))
+        self.gamePlug.send(FireShotMsg(msg.player.id, msg.angle, msg.xpos, msg.ypos, msg.type))        
 
-    def sendZoneTagged(self, zone, player, team):
+    @handler(GrenadeExploded, orderPlug)    
+    def sendGrenadeExploded(self, msg):
+        self.gamePlug.send(GrenadeExplodedMsg(msg.grenade.player.id, msg.grenade.gr.pos[0], msg.grenade.gr.pos[1]))
+
+    @handler(TaggedZone, orderPlug)
+    def sendZoneTagged(self, msg):
         '''Sends the server a message indicating that the zone has been tagged
         by the given player'''
-        if team is None:
+        player = msg.player
+        zone = msg.zone
+        if player is None:
             teamId = '\x00'
+            pId = 0
         else:
-            teamId = team.id
-        msg = TaggingZoneMsg(zone.id, player.id, teamId)
+            teamId = player.team.id
+            pId = player.id
+        msg = TaggingZoneMsg(zone.id, pId, teamId)
         self.gamePlug.send(msg)
 
     def receiveZoneTagged(self, team, player, zone):
         '''Tells the universe object that a zone has been tagged'''
         # TODO: receive turret deaths even when zone is rendered neutral
+        
         zone.tag(team, player)
-        self.viewPlug.send(TaggedZone(zone, player))
 
-    def sendPlayerKilled(self, player, killer, shot):
+    @handler(PlayerKilled, orderPlug)
+    def sendPlayerKilled(self, msg):
         '''Sends the server a message indicating that a player has been
         killed by a certain shot'''
-        self.gamePlug.send(KillingMsg(player.id, killer.id, shot.id,
-                player.pos[0], player.pos[1]))
+        target = msg.target
+        self.gamePlug.send(KillingMsg(target.id, msg.killer.id, msg.shot.id,
+                target.pos[0], target.pos[1]))
 
-    def receivePlayerKilled(self, pId, kId, sId):
-        '''Tells the universe that a certain player has been killed, who they
-        were killed by, using which shot'''
-        # NOTE: This is also called when a turret is killed due to zone capture
-        player = self.world.playerWithId[pId]
-        killer = self.world.playerWithId[kId]
-        if sId:
-            try:
-                shot = killer.shots[sId]
-            except KeyError:
-                logging.writeException()
-                shot = None
-        else:
-            shot = None
-        self.world.killPlayer(player, killer, shot)
-        self.viewPlug.send(PlayerKilled(player, killer))
-
-    def shotDestroyed(self, shot):
+    @handler(KillShot, orderPlug)    
+    def shotDestroyed(self, msg):
         '''Tells the network that a shot has been destroyed (likely by
         colliding with a turretted player)'''
-        msg = KillShotMsg(shot.originatingPlayer.id, shot.id)
-        self.gamePlug.send(msg)
-    
-    def sendPlayerRespawn(self, ghost):
+        msg2 = KillShotMsg(msg.player.id, msg.shot.id)
+        self.gamePlug.send(msg2)
+
+    @handler(PlayerRespawn, orderPlug)    
+    def sendPlayerRespawn(self, msg):
         '''Tells the network that a player has respawned at the given
         position'''
-        if self.world.gameState == GameState.PreGame:
-            return
-        self.gamePlug.send(RespawnMsg(ghost.id, ghost.currentZone.id))
-        
-    @handler(RespawnMsg, gamePlug)
-    def receiveRespawnMsg(self, msg):
-        player = self.world.playerWithId[msg.playerId]
-        zone = self.world.zoneWithId[msg.zoneId]
-        self.world.respawn(player, zone)
-        self.viewPlug.send(PlayerRespawn(player, zone))
+        self.gamePlug.send(RespawnMsg(msg.player.id, msg.zone.id))
 
-    def startTransaction(self, upgrade, player, stars):
+    @handler(StartingTransactionMsg, orderPlug)
+    def startTransaction(self, msg):
         '''Initiates a transaction to purchase an upgrade'''
-        self.gamePlug.send(StartingTransactionMsg(player.team.id,
-                player.id, stars, upgrade.upgradeType))
+        self.gamePlug.send(msg)
         
-
-    def addStars(self, team, player, stars):
+    @handler(AddingStarsMsg, orderPlug)
+    def addStars(self, msg):
         '''Informs the network of a player's star-adding'''
-        self.gamePlug.send(AddingStarsMsg(team.id, player.id, stars))
+        self.gamePlug.send(msg)
 
-    def abandonTransaction(self, team):
+    @handler(AbandonTransaction, orderPlug)
+    def abandonTransaction(self, msg):
         '''Informs the network that a transaction has been abandoned by the
         initiating player.'''
-        self.gamePlug.send(AbandonTransactionMsg(team.id))
+        self.gamePlug.send(AbandonTransactionMsg(msg.transaction.purchasingTeam.id, msg.reason))
 
-    def useUpgrade(self, player):
-        '''Informs the network that an update has expired, or that
-        its owning player has died'''
+    @handler(UseUpgrade, orderPlug)
+    def useUpgrade(self, msg):
+        player = msg.player
         self.gamePlug.send(UseUpgradeMsg(player.id, player.pos[0],
                 player.pos[1], player.currentZone.id,
                 player.upgrade.upgradeType))
-
-    def upgradeDelete(self, player):
+        
+    @handler(DeleteUpgrade, orderPlug)
+    def upgradeDelete(self, msg):
         '''Informs the server that the upgrade has been prematurely deleted'''
-        self.gamePlug.send(DeleteUpgradeMsg(player.id))
+        self.gamePlug.send(DeleteUpgradeMsg(msg.player.id))
 
     def requestTransactionInfo(self, team):
         self.gamePlug.send(RequestTransactionMsg(team.id))
-    
+
+    # TODO: Once we use TCP for transaction stuff, this should theoretically not be needed.    
     def validateTransaction(self, team, contributors, totalStars,
                             player, timeLeft, upgrade):
         '''Validates a current transaction, making sure that the player
@@ -509,18 +492,15 @@ class Client(Component):
         transaction = team.currentTransaction
             
         if transaction is None:
-            transaction = transactionModule.Transaction(team, player,
-                                                       upgrade, timeLeft)
-            self.viewPlug.send(StartedTransaction(transaction))
-            transaction.addStars(player, contributors[player])
+            self.worldPlug.send(StartedTransactionMsg(team.id, player.id, contributors[player], upgrade.upgradeType, timeLeft))
         else:
             if contributors == transaction.contributions and \
                player == transaction.purchasingPlayer:
                 # We're happy
                 pass
             else:
-                transaction.contributions = contributors
-                self.viewPlug.send(TransactionChanged(transaction))
+                # FIXME: Yucky hack.
+                self.world.updateContributions(transaction, contributions)
                 
     def requestPlayers(self):
         '''Request the list of players currently in the game.'''
@@ -538,9 +518,10 @@ class Client(Component):
         '''Request the current upgrades that all players have.'''
         self.gamePlug.send(RequestUpgradesMsg())
 
-    def sendZoneChange(self, player, zone):
+    @handler(ZoneChangeMsg, orderPlug)
+    def sendZoneChange(self, msg):
         '''Inform the server that a local player has changed zones.'''
-        self.gamePlug.send(ZoneChangeMsg(player.id, zone.id))
+        self.gamePlug.send(msg)
         
     def sendChat(self, text, sendingPlayer, target=None):
         '''Send a chat message to the server. The server will send it out
@@ -564,21 +545,15 @@ class Client(Component):
 
         self.gamePlug.send(ChatMsg(sendingPlayer.id, kind, targetId,
             text.encode()))
-        self.viewPlug.send(OwnChat(sendingPlayer.team, target, text))
+        self.tempWorldPlug.send(OwnChat(sendingPlayer.team, target, text))
 
-    
-    def receiveChat(self, text, sender, private):
-        '''We have received a chat from someone in the game.'''
-        self.viewPlug.send(Chat(sender, private, text))
+    @handler(SetCaptain, orderPlug)
+    def setAsCaptain(self, msg):
+        self.gamePlug.send(SetCaptainMsg(msg.captain.id))
 
-    def setAsCaptain(self, player):
-        if player.team.captain:
-            return
-        self.gamePlug.send(SetCaptainMsg(player.id))
-
-    def teamReady(self, player):
-        if player.team.captain == player and not player.team.ready:
-            self.gamePlug.send(TeamIsReadyMsg(player.id))
+    @handler(TeamIsReady, orderPlug)
+    def teamReady(self, msg):
+        self.gamePlug.send(TeamIsReadyMsg(msg.player.id))
 
     def madeCaptain(self, pId):
         player = self.world.playerWithId[pId]
@@ -587,21 +562,6 @@ class Client(Component):
     #########################
     # Message handling.
 
-    @handler(NotifyUDPStatus, gamePlug)
-    def sendToView(self, msg):
-        self.viewPlug.send(msg)
-        
-    @handler(AddPlayerMsg, gamePlug)
-    def receiveAddPlayerMsg(self, msg):
-        'Server announcing new player.'
-        team = self.world.teamWithId[msg.teamId]
-        zone = self.world.zoneWithId[msg.zoneId]
-
-        # Create the player.
-        nick = msg.nick.decode()
-        
-        self.world.addPlayer(nick, team, msg.playerId, zone, dead = True)
-
     @handler(GivePlayerMsg, gamePlug)
     def receiveGivePlayerMsg(self, msg):
         # Server gives this player to me.
@@ -615,31 +575,25 @@ class Client(Component):
             self.cancelingPlayers.remove((player.team, player.nick))
         else:
             try:
-                deferred = self.waitingPlayers[player.team, player.nick]
+                if msg.auto == 1:
+                    deferred = self.waitingPlayers[None, player.nick]
+                else:
+                    deferred = self.waitingPlayers[player.team, player.nick]
+                    
                 # Fire the deferred's callbacks.
                 deferred.callback(('success', player))
             except KeyError:
                 logging.writeException()
-                
-    @handler(RemovePlayerMsg, gamePlug)
-    def receiveRemovePlayerMsg(self, msg):
-        # Server says this player has left the game.
-        try:
-            player = self.world.playerWithId[msg.playerId]
-        except KeyError:
-            # TODO: Handle this sensibly.
-            logging.writeException()
-            return
-        
-        player.removeFromGame()
-        self.viewPlug.send(RemovePlayer(player))
 
     @handler(CannotCreatePlayerMsg, gamePlug)
     def receiveCannotCreatePlayerMsg(self, msg):
         # Find the deferred and call its ErrBack.
         try:
-            deferred = self.waitingPlayers[
-                self.world.teamWithId[msg.teamId], msg.nick]
+            if msg.auto == 1:
+                deferred = self.waitingPlayers[None, msg.nick]
+            else:
+                deferred = self.waitingPlayers[
+                    self.world.teamWithId[msg.teamId], msg.nick]
         except KeyError:
             logging.writeException()
             return
@@ -650,34 +604,10 @@ class Client(Component):
             deferred.callback(['over'])
         elif msg.reasonId == 'W':
             deferred.callback(['wait', str(round(msg.waitTime, 1))])
+        elif msg.reasonId == 'N':
+            deferred.callback(['nick'])
         else:
             deferred.callback(['unknown reason code: %s' % msg.reasonId])
-        
-    @handler(GameStartMsg, gamePlug)
-    def receiveGameStartMsg(self, msg):
-        # The game has progressed from its pre-game state, to being
-        # in progress.
-        self.world.gameStart(msg.timeLimit)
-        self.viewPlug.send(GameStart(msg.timeLimit))
-        
-    @handler(GameOverMsg, gamePlug)
-    def receiveGameOverMsg(self, msg):
-        if msg.teamId == '\x00':
-            winningTeam = None
-        else:
-            winningTeam = self.world.teamWithId[msg.teamId]
-
-        self.viewPlug.send(GameOver(winningTeam, msg.timeOver))
-
-        self.world.gameOver()
-        
-    @handler(SetGameModeMsg, gamePlug)
-    def receiveSetGameModeMsg(self, msg):
-        self.world.setGameMode(msg.gameMode.decode())
-
-    @handler(SetTeamNameMsg, gamePlug)
-    def receiveSetTeamNameMsg(self, msg):
-        self.world.teamWithId[msg.teamId].teamName = msg.name.decode()
 
     def _leave(self, playerId):
         self.gamePlug.send(LeaveRequestMsg(playerId))
@@ -690,147 +620,6 @@ class Client(Component):
         except ValueError:
             pass
 
-    @handler(ShotFiredMsg, gamePlug)
-    def receiveShotFiredMsg(self, msg):
-        # Shot fired.
-        pos = (msg.xpos, msg.ypos)
-        self.receiveShotFired(pos, msg.angle,
-                msg.playerId, msg.shotId, msg.type in ('T', 'G'), msg.type == 'R')
-        
-        # FIXME: Once the grenade no longer uses the Shot message, this
-        #       line can be put back in. Currently this line will cause
-        #       an apparent glitch when a grenade explodes.
-        # self.positionUpdate(msg.playerId, pos, angle = msg.angle)
-        
-    @handler(KillShotMsg, gamePlug)
-    def receiveKillShotMsg(self, msg):
-        shot = self.world.shotWithId(msg.playerId, msg.shotId)
-        if shot is not None:
-            shot.kill()
-
-    @handler(SetCaptainMsg, gamePlug)
-    def receiveSetCaptainMsg(self, msg):
-        player = self.world.playerWithId[msg.playerId]
-        player.team.captain = player
-        self.viewPlug.send(SetCaptain(player))
-
-    @handler(TeamIsReadyMsg, gamePlug)
-    def receiveTeamIsReadyMsg(self, msg):
-        player = self.world.playerWithId[msg.playerId]
-        player.team.ready = True
-        self.viewPlug.send(TeamIsReady(player))
-
-    @handler(KilledMsg, gamePlug)
-    def receiveKilledMsg(self, msg):
-        # Player killed.
-        self.receivePlayerKilled(msg.targetId, msg.killerId, msg.shotId)
-        self.positionUpdate(msg.targetId, (msg.xPos, msg.yPos))
-        killer = self.world.playerWithId[msg.killerId]
-        self.world.updateStars(killer, msg.stars)
-
-    @handler(PlayerUpdateMsg, gamePlug)
-    def receivePlayerUpdateMsg(self, msg):
-        # Receive a player update
-        try:
-            player = self.world.playerWithId[msg.playerId]
-        except:
-            # Mustn't have that info yet
-            return
-        if not player.local:            
-            values = expand_boolean(getattr(msg, 'keys'))
-            opts = ['left', 'right', 'jump', 'down']
-            
-            for i in range(4):
-                player.updateState(opts[i], bool(values[i]))
-                
-            hasUpgrade = values[4]
-            ghost = values[5]
-            
-            if (not ((hasUpgrade and player.upgrade) or
-                   not (hasUpgrade or player.upgrade))):
-                self.requestUpgrades()
-                
-            if ghost != player.ghost:
-                if player.ghost:
-                    player.respawn()
-                else:
-                    player.die()
-            
-            self.positionUpdate(msg.playerId, (msg.xPos, msg.yPos),
-                    msg.yVel, angle=msg.angle, thrust=msg.ghostThrust)
-
-    @handler(TaggedZoneMsg, gamePlug)
-    def receiveTaggedZoneMsg(self, msg):
-        # Zone has been tagged.
-
-        zone = self.world.zoneWithId[msg.zoneId]
-        team = self.world.teamWithId[msg.teamId]
-        if team is None:
-            self.receiveZoneTagged(None, None, zone)
-        else:
-            player = self.world.playerWithId[msg.playerId]
-            self.receiveZoneTagged(team, player, zone)
-            if not team.validate(msg.teamOrbCount):
-                self.requestZones()
-            if msg.killedTurret:
-                self.receivePlayerKilled(msg.turretId, msg.playerId, None)
-                
-            self.world.updateStars(player, msg.playerStarCount)
-
-    @handler(StartedTransactionMsg, gamePlug)
-    def receiveStartedTransactionMsg(self, msg):
-        # A transaction has begun
-        team = self.world.teamWithId[msg.teamId]
-        player = self.world.playerWithId[msg.playerId]
-        upgrade = upgradeOfType[msg.upgradeType]
-        transaction = transactionModule.Transaction(team, player,
-                                                   upgrade, msg.timeLeft)
-        transaction.addStars(player, msg.numStars)
-        self.viewPlug.send(StartedTransaction(transaction))
-            
-    @handler(DeleteUpgradeMsg, gamePlug)
-    def receiveDeleteUpgradeMsg(self, msg):
-        # A player's upgrade is deleted
-        player = self.world.playerWithId[msg.playerId]
-        if player.upgrade is not None:
-            self.viewPlug.send(DeleteUpgrade(player))
-            player.upgrade.clientDelete()
-
-    @handler(AddedStarsMsg, gamePlug)
-    def receiveAddedStarsMsg(self, msg):
-        # A player has added star(s) to a transaction
-        team = self.world.teamWithId[msg.teamId]
-        player = self.world.playerWithId[msg.playerId]
-        if not self.world.validateTransaction(team, 's', msg.totalStars - msg.numStars):
-            self.requestTransactionInfo(team)
-            return
-        self.world.addStarsToTransaction(team, player, msg.numStars)
-
-    @handler(AbandonTransactionMsg, gamePlug)
-    def receiveAbandonTransactionMsg(self, msg):
-        # A transaction has been abandoned
-        team = self.world.teamWithId[msg.teamId]
-        if not self.world.validateTransaction(team, 'a', None):
-            self.requestTransactionInfo(team)
-            return
-        self.world.abandonTransaction(team)
-            
-    @handler(UseUpgradeMsg, gamePlug)
-    def receiveUseUpgradeMsg(self, msg):
-        # A player is using an upgrade
-        player = self.world.playerWithId[msg.playerId]
-        self.positionUpdate(msg.playerId, (msg.xPos, msg.yPos))
-        if not (player.upgrade and player.upgrade.upgradeType ==
-                msg.upgradeType and not player.upgrade.inUse):
-            # Correct our known data
-            player.upgrade = upgradeOfType[msg.upgradeType](player)
-        if isinstance(player.upgrade, Turret):
-            zone = self.world.zoneWithId[msg.zoneId]
-            if zone.turretedPlayer:
-                # We think there's already a turreted player within.
-                self.requestUpgrades()
-        player.upgrade.clientUse()
-        self.viewPlug.send(UseUpgrade(player))
 
     @handler(ValidateTransactionMsg, gamePlug)
     def receiveValidateTransactionMsg(self, msg):
@@ -857,43 +646,16 @@ class Client(Component):
         self.validateTransaction(team, contributors, totalStars,
                                         player, msg.timeLeft, upgrade)
 
-    @handler(TransactionCompleteMsg, gamePlug)
-    def receiveTransactionCompleteMsg(self, msg):
-        # A transaction has been completed
-        # Double checks that all the transaction info is correct, then
-        # completes the transaction.
-        team = self.world.teamWithId[msg.teamId]
-        transaction = team.currentTransaction
-
-        if transaction is None:
-            # Get the correct data
-            try:
-                player = self.world.playerWithId[msg.playerId]
-            except KeyError:
-                logging.writeException()
-                self.requestPlayers()
-                return
-
-            upgrade = upgradeOfType[msg.upgradeType]
-            team.currentTransaction = transaction = \
-                            transactionModule.Transaction(team,
-                            player, upgrade)
-            
-            # Add the full complement of stars
-            # TODO: I'm not sure whether this is right:
-            transaction.addStars(player, upgrade.requiredStars)
-            # We don't know for certain who donated how many stars.
-            self.viewPlug.send(StartedTransaction(transaction))
-
-        self.world.completeTransaction(team)
-
+    # ##################################################
+    # TODO: The following three functions should be able
+    # to be removed once universe.py is used server-side
     @handler(RequestPlayerUpdateMsg, gamePlug)
     def receiveRequestPlayerUpdateMsg(self, msg):
         # Received from the server when it wants this client to send a player
         # update
         player = self.world.playerWithId[msg.playerId]
         if player.local:
-            self.sendPlayerUpdate(player)
+            player.sendPlayerUpdate()
 
     @handler(RequestZoneUpdateMsg, gamePlug)
     def receiveRequestZoneUpdateMsg(self, msg):
@@ -911,6 +673,9 @@ class Client(Component):
             zone = self.world.zoneWithId[msg.zoneId]
             player.changeZone(zone)
 
+    # ##################################################
+    # ##################################################
+
     @handler(ValidateZoneMsg, gamePlug)
     def receiveValidateZoneMsg(self, msg):
         zone = self.world.zoneWithId[msg.zoneId]
@@ -929,7 +694,6 @@ class Client(Component):
     @handler(ValidatePlayersMsg, gamePlug)
     def receiveValidatePlayersMsg(self, msg):
         # Get rid of any non-existant players
-        playersToKick = []
         for player in list(self.world.playerWithId.itervalues()):
             if player.id not in msg.playerIds:
                 player.removeFromGame()
@@ -942,12 +706,12 @@ class Client(Component):
             # TODO: It might be worth checking that the other
             # information is correct too (current zone)
         else:
-            team = self.world.teamWithId[msg.teamId]
-            zone = self.world.zoneWithId[msg.zoneId]
-            self.world.addPlayer(msg.nick.decode(),
-                    team, msg.playerId, zone, dead=msg.isDead)
+            self.worldPlug.send(AddPlayerMsg(msg.playerId, msg.teamId, 
+                    msg.zoneId, msg.isDead, msg.nick))
             player = self.world.playerWithId[msg.playerId]
-        self.world.updateStars(player, msg.stars)
+
+        # TODO: This is non-ideal
+        player.stars = msg.stars
 
     @handler(ValidateUpgradesMsg, gamePlug)
     def receiveValidateUpgradesMsg(self, msg):
@@ -1023,31 +787,6 @@ class Client(Component):
             # there's no point in wasting the server's time
             self.requestTransactionInfo(team)
 
-    @handler(ChatMsg, gamePlug)
-    def receiveChatMsg(self, msg):
-        # A chat is received
-        sender = self.world.playerWithId[msg.senderId]
-        if msg.kind == 't':     # Team
-            team = self.world.teamWithId[msg.targetId]
-            self.receiveChat(msg.text.decode(), sender, team)
-        elif msg.kind == 'p':   # Private
-            print 'Received private chat for player %d' % (ord(msg.targetId))
-            receiver = self.world.playerWithId[msg.targetId]
-            # Find the address to send it to:
-            self.receiveChat(msg.text.decode(), sender, receiver)
-        elif msg.kind == 'o':   # Open
-            self.receiveChat(msg.text.decode(), sender, None)
-        else:
-            print 'Client: received unknown chat kind: %r' % (msg.kind,)
-
-    @handler(ChatFromServerMsg, gamePlug)
-    def receiveChatFromServerMsg(self, msg):
-        self.viewPlug.send(ChatFromServer(msg.text.decode()))
-
-    @handler(StartingSoonMsg, gamePlug)
-    def receiveStartingSoonMsg(self, msg):
-        self.viewPlug.send(StartingSoon(msg.delay))
-
     @handler(GotSettings, gamePlug)
     def gotSettings(self, msg):
         'Called when the server initially sends its settings.'
@@ -1059,7 +798,7 @@ class Client(Component):
             try:
                 layouts.append(self.layoutDatabase.getLayout(filename,
                                                              checksum))
-            except mapLayout.ChecksumError, E:
+            except mapLayout.ChecksumError:
                 self.requestMapBlock(filename,len(layouts))
                 # Put a placeholder in
                 layouts.append(None)
@@ -1085,18 +824,15 @@ class Client(Component):
             halfMapWidth = self.serverSettings['halfMapWidth']
             mapHeight = self.serverSettings['mapHeight']
 
-            self.world = universe.Universe(halfMapWidth, mapHeight)
-            self.world.onZoneTagged.addListener(self.sendZoneTagged)
-            self.world.onPlayerKilled.addListener(self.sendPlayerKilled)
-            self.world.onShotDestroyed.addListener(self.shotDestroyed)
-            self.world.onZoneChanged.addListener(self.sendZoneChange)
-            self.world.onPlayerUpdate.addListener(self.sendPlayerUpdate)
-            self.world.onShotFired.addListener(self.sendShotFired)
-            self.world.onPlayerRespawn.addListener(self.sendPlayerRespawn)
-            self.world.onAbandonTransaction.addListener(self.abandonTransaction)
-            self.world.onUpgradeReqDel.addListener(self.upgradeDelete)
-            self.world.onUpgradeReqUse.addListener(self.useUpgrade)
-            self.world.onGrenadeExploded.addListener(self.sendGrenadeExploded)
+            self.world = universe.Universe(self.app, halfMapWidth, mapHeight)
+            
+            # For being told ordered to do when the universe requests something
+            self.orderPlug.connectPlug(self.world.requestPlug)
+            
+            # For telling the universe what to do
+            translator = MessageTranslator_N2U(self.world)
+            self.tempWorldPlug.connectPlug(translator.receiverPlug)
+            self.worldPlug.connectPlug(self.world.orderPlug)
             
             for zone in self.world.zones: 
                 zone.mapBlockList = [] 
@@ -1188,7 +924,7 @@ class NetworkClient(Client):
         self._errorConnecting(msg.reason)
 
     def _errorConnecting(self, reason):
-        self.connectingDeferred.errback(UnableToConnect(reason))
+        self.connectingDeferred.errback(Failure(UnableToConnect(reason)))
         self.nethandler = None
 
     def _cancelConnecting(self, error):
@@ -1201,7 +937,7 @@ class NetworkClient(Client):
         self.clientConnectionLost()
         return error
 
-    def cancelConnecting(self, error = None):
+    def cancelConnecting(self, error=None):
         print 'Client: Attempting to cancel connection'
         if self.connectingDeferred is None:
             raise ValueError('Cannot cancel connecting when not connecting.')
