@@ -28,7 +28,6 @@ from trosnoth.src.gui.fonts import font
 from trosnoth.src.gui.common import *
 from trosnoth.src.gui.framework.dialogbox import MoveableBox, DialogResult, DialogBoxAttachedPoint
 import trosnoth.src.gui.framework.framework as framework
-from trosnoth.src.trosnothgui.ingame import colours
 
 class ChatBox(MoveableBox):
     def __init__(self, app, world, gameViewer):
@@ -50,7 +49,7 @@ class ChatBox(MoveableBox):
         
         self.helpText = TextElement(self.app, 'Press CTRL to broadcast', self.app.screenManager.fonts.defaultTextBoxFont,
                                         Location(DialogBoxAttachedPoint(self, ScaledSize(0, 50), 'midtop'), 'midtop'),
-                                        colours.black)
+                                        app.theme.colours.black)
 
         self.elements = [self.input, self.helpText]
         self.setFocus(self.input)
@@ -77,6 +76,7 @@ class ChatBox(MoveableBox):
         self.updateColours()
 
     def updateColours(self):
+        colours = self.app.theme.colours
         titleColour = colours.white
         if self.chatMode == 'team':
             if self.player.team.id == 'A':
