@@ -26,7 +26,6 @@ from trosnoth.src.model.map import MapLayout
 from trosnoth.src.model import mapblocks
 import trosnoth.src.gui.framework.framework as framework
 from trosnoth.src.gui.framework.basics import Animation
-from trosnoth.src.trosnothgui.ingame import colours
 
 # TODO: Get a full-screen minimap upon a hotkey.
 
@@ -83,6 +82,7 @@ class MiniMap(framework.Element):
         '''Draws the current state of the universe at the current viewing
         location on the screen.  Does not call pygame.display.flip() .'''
 
+        colours = self.app.theme.colours
         pygame.draw.rect(screen, colours.black, self.sRect, 0)
 
         # If disrupted, draw static 95% of the time
@@ -111,9 +111,11 @@ class MiniMap(framework.Element):
 
                 else:
                     if player.ghost:
-                        clr = player.team.miniMapGhostColour
+                        clr = self.app.theme.colours.miniMapGhostColour(
+                                player.team)
                     else:
-                        clr = player.team.miniMapPlayerColour
+                        clr = self.app.theme.colours.miniMapPlayerColour(
+                                player.team)
                     radius = 2
                 playerPos = [int(playerPos[i]) for i in (0,1)]
                 pygame.draw.circle(screen, clr, playerPos, radius)
@@ -182,7 +184,8 @@ class MiniMap(framework.Element):
             posToDraw[1] += self.interfaceRect.height
             if posToDraw[1] > self.sRect.bottom:
                 break
-        pygame.draw.rect(screen, colours.minimapBorder, self.sRect, 2)
+        pygame.draw.rect(screen, self.app.theme.colours.minimapBorder,
+                self.sRect, 2)
 
                                                
     def updateFocus(self):
@@ -205,6 +208,7 @@ class MiniMap(framework.Element):
 
     def createDisrupted(self):
         screens = []
+        colours = self.app.theme.colours
         for a in range(0,4):
             screen = (pygame.surface.Surface(self.sRect.size))
             x = y = yy = 0
@@ -226,5 +230,7 @@ class MiniMap(framework.Element):
 
     def drawDisrupted(self, screen):
         screen.blit(self.disruptAnimation.getImage(), self.sRect.topleft)
+        pygame.draw.rect(screen, self.app.theme.colours.minimapBorder,
+                self.sRect, 2)
                 
         
