@@ -15,7 +15,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-from trosnoth.src.trosnothgui.pregame import colours
+import os
+
+from trosnoth.src.trosnothgui.pregame.common import button
 from trosnoth.src.gui.framework import prompt
 from trosnoth.src.gui.framework.tab import Tab
 from trosnoth.src.gui.framework.elements import TextElement, TextButton
@@ -24,15 +26,18 @@ from trosnoth.src.gui.common import *
 from trosnoth.src.gui import keyboard
 from trosnoth.src import keymap
 from trosnoth.data import getPath, user, makeDirs
-import os
+from trosnoth.src.utils.event import Event
 
 
 class KeymapTab(Tab):
 
-    def __init__(self, app, startupInterface):
+    def __init__(self, app, onClose=None):
         super(KeymapTab, self).__init__(app, 'Controls')
-        self.startupInterface = startupInterface
-        button = startupInterface.button
+        self.font = app.screenManager.fonts.bigMenuFont
+
+        self.onClose = Event()
+        if onClose is not None:
+            self.onClose.addListener(onClose)
 
         # Break things up into categories
         movement = ['left', 'right', 'jump', 'down']
@@ -41,7 +46,7 @@ class KeymapTab(Tab):
                     'abandon', 'captain', 'team ready']
         misc =     ['chat']
         upgrades = ['turret', 'shield', 'minimap disruption', 'phase shift',
-                    'grenade']
+                    'grenade', 'ricochet']
         display =  ['leaderboard', 'toggle interface']
 
         actionNames = { 'abandon': 'Abandon upgrade',
@@ -58,6 +63,7 @@ class KeymapTab(Tab):
                         'minimap disruption': 'Minimap Disruption',
                         'phase shift': 'Phase Shift',
                         'respawn': 'Respawn',
+                        'ricochet': 'Ricochet',
                         'right': 'Move right',
                         'shield': 'Shield',
                         'status bar': 'Status bar',
@@ -71,7 +77,7 @@ class KeymapTab(Tab):
         # Organise the categories by column
         self.layout = [ [movement, menus, misc], [actions], [upgrades, display] ]
 
-        self.errorInfo = TextElement(self.app, '', startupInterface.font,
+        self.errorInfo = TextElement(self.app, '', self.font,
                                  ScaledLocation(512, 580, 'center'))
         self.text = [
                      self.errorInfo
@@ -92,7 +98,7 @@ class KeymapTab(Tab):
                     self.text.append(TextElement(self.app, actionNames[action],
                                     keymapFont,
                                     ScaledLocation(xPos, yPos,'topright'),
-                                    colours.headingColour
+                                    self.app.theme.colours.headingColour
                                     ))
 
                     # Create input box
@@ -108,13 +114,13 @@ class KeymapTab(Tab):
             xPos += 320         # Between columns
 
         self.elements = self.text + self.inputLookup.values() + \
-                        [button('restore default controls',
+                        [button(app, 'restore default controls',
                                 self.restoreDefaults,
                                 (0, -125), 'midbottom'),
-                         button('save',
+                         button(app, 'save',
                                 self.saveSettings,
                                 (-100, -75), 'midbottom'),
-                         button('cancel',
+                         button(app, 'cancel',
                                 self.cancel,
                                 (100, -75), 'midbottom'),
                          ]
@@ -188,7 +194,7 @@ class KeymapTab(Tab):
         if len(emptyBoxes) > 0:
             self.populateInputs()
             for box in emptyBoxes:
-                box.backColour = colours.invalidDataColour
+                box.backColour = self.app.theme.colours.invalidDataColour
 
             self.incorrectInput('Warning: some actions have no key', (192, 0, 0))
         else:
@@ -197,7 +203,7 @@ class KeymapTab(Tab):
     def incorrectInput(self, string, colour):
         self.errorInfo.setColour(colour)
         self.errorInfo.setText(string)
-        self.errorInfo.setFont(self.startupInterface.font)
+        self.errorInfo.setFont(self.font)
 
     def cancel(self):
         self.populateInputs()
@@ -206,4 +212,4 @@ class KeymapTab(Tab):
     def mainMenu(self):
         self.incorrectInput('', (0,0,0))
         self.clearBackgrounds()
-        self.startupInterface.mainMenu()
+        self.onClose.execute()
