@@ -22,19 +22,22 @@ from trosnoth.src.gui.framework.checkbox import CheckBox
 from trosnoth.src.gui.framework.tab import Tab
 from trosnoth.src.gui.framework.slider import Slider
 import trosnoth.src.gui.framework.prompt as prompt
-import trosnoth.src.trosnothgui.pregame.colours as colours
+from trosnoth.src.trosnothgui.pregame.common import button
 from trosnoth.src.gui.common import *
+from trosnoth.src.utils.event import Event
 
 
 class SoundSettingsTab(Tab, framework.CompoundElement):
-    def __init__(self, app, startupInterface):
+    def __init__(self, app, onClose=None):
         super(SoundSettingsTab, self).__init__(app, "Sounds")
-        self.startupInterface = startupInterface
 
-        button = startupInterface.button
+        self.onClose = Event()
+        if onClose is not None:
+            self.onClose.addListener(onClose)
 
         font = self.app.screenManager.fonts.bigMenuFont
         smallNoteFont = self.app.screenManager.fonts.smallNoteFont
+        colours = app.theme.colours
 
         text = [
             TextElement(self.app, 'Music Volume', font,
@@ -53,6 +56,7 @@ class SoundSettingsTab(Tab, framework.CompoundElement):
                 ScaledArea(450, 280, 400, 40))
         onSlide = lambda volume: musicVolumeLabel.setText("%d" % volume)
         self.musicVolumeSlider.onSlide.addListener(onSlide)
+        self.musicVolumeSlider.onValueChanged.addListener(onSlide)
         self.musicVolumeSlider.setVal(initVolume)
         
         self.musicBox = CheckBox(self.app, ScaledLocation(450, 360),
@@ -60,8 +64,8 @@ class SoundSettingsTab(Tab, framework.CompoundElement):
                 initValue = app.soundSettings.musicEnabled)
 
         self.buttons = [
-            button('save', self.saveSettings, (-100, -75), 'midbottom'),
-            button('cancel', startupInterface.mainMenu, (100, -75),
+            button(app, 'save', self.saveSettings, (-100, -75), 'midbottom'),
+            button(app, 'cancel', self.onClose.execute, (100, -75),
                 'midbottom')
         ]
         
@@ -78,7 +82,7 @@ class SoundSettingsTab(Tab, framework.CompoundElement):
         ss.save()
         ss.apply()
 
-        self.startupInterface.mainMenu()
+        self.onClose.execute()
 
     def getValues(self):
 
